@@ -31,7 +31,7 @@ class DeviantartExtractor(Extractor):
     root = "https://www.deviantart.com"
     directory_fmt = ("{category}", "{username}")
     filename_fmt = "{category}_{index}_{title}.{extension}"
-    cookies_domain = None
+    cookies_domain = ".deviantart.com"
     cookies_names = ("auth", "auth_secure", "userinfo")
     _last_request = 0
 
@@ -399,9 +399,9 @@ class DeviantartExtractor(Extractor):
 
     def _textcontent_to_html(self, deviation, content):
         html = content["html"]
-        markup = html["markup"]
+        markup = html.get("markup")
 
-        if not markup.startswith("{"):
+        if not markup or markup[0] != "{":
             return markup
 
         if html["type"] == "tiptap":
@@ -440,7 +440,8 @@ class DeviantartExtractor(Extractor):
                     html.append("text-align:")
                     html.append(attrs["textAlign"])
                     html.append(";")
-                html.append('margin-inline-start:0px">')
+                self._tiptap_process_indentation(html, attrs)
+                html.append('">')
 
                 for block in children:
                     self._tiptap_process_content(html, block)
@@ -451,6 +452,41 @@ class DeviantartExtractor(Extractor):
         elif type == "text":
             self._tiptap_process_text(html, content)
 
+        elif type == "heading":
+            attrs = content["attrs"]
+            level = str(attrs.get("level") or "3")
+
+            html.append("<h")
+            html.append(level)
+            html.append(' style="text-align:')
+            html.append(attrs.get("textAlign") or "left")
+            html.append('">')
+            html.append('<span style="')
+            self._tiptap_process_indentation(html, attrs)
+            html.append('">')
+            self._tiptap_process_children(html, content)
+            html.append("</span></h")
+            html.append(level)
+            html.append(">")
+
+        elif type in ("listItem", "bulletList", "orderedList", "blockquote"):
+            c = type[1]
+            tag = (
+                "li" if c == "i" else
+                "ul" if c == "u" else
+                "ol" if c == "r" else
+                "blockquote"
+            )
+            html.append("<" + tag + ">")
+            self._tiptap_process_children(html, content)
+            html.append("</" + tag + ">")
+
+        elif type == "anchor":
+            attrs = content["attrs"]
+            html.append('<a id="')
+            html.append(attrs.get("id") or "")
+            html.append('" data-testid="anchor"></a>')
+
         elif type == "hardBreak":
             html.append("<br/><br/>")
 
@@ -468,6 +504,44 @@ class DeviantartExtractor(Extractor):
             html.append(user)
             html.append('</a>')
 
+        elif type == "da-gif":
+            attrs = content["attrs"]
+            width = str(attrs.get("width") or "")
+            height = str(attrs.get("height") or "")
+            url = text.escape(attrs.get("url") or "")
+
+            html.append('<div data-da-type="da-gif" data-width="')
+            html.append(width)
+            html.append('" data-height="')
+            html.append(height)
+            html.append('" data-alignment="')
+            html.append(attrs.get("alignment") or "")
+            html.append('" data-url="')
+            html.append(url)
+            html.append('" class="t61qu"><video role="img" autoPlay="" '
+                        'muted="" loop="" style="pointer-events:none" '
+                        'controlsList="nofullscreen" playsInline="" '
+                        'aria-label="gif" data-da-type="da-gif" width="')
+            html.append(width)
+            html.append('" height="')
+            html.append(height)
+            html.append('" src="')
+            html.append(url)
+            html.append('" class="_1Fkk6"></video></div>')
+
+        elif type == "da-video":
+            src = text.escape(content["attrs"].get("src") or "")
+            html.append('<div data-testid="video" data-da-type="da-video" '
+                        'data-src="')
+            html.append(src)
+            html.append('" class="_1Uxvs"><div data-canfs="yes" data-testid="v'
+                        'ideo-inner" class="main-video" style="width:780px;hei'
+                        'ght:438px"><div style="width:780px;height:438px">'
+                        '<video src="')
+            html.append(src)
+            html.append('" style="width:100%;height:100%;" preload="auto" cont'
+                        'rols=""></video></div></div></div>')
+
         else:
             self.log.warning("Unsupported content type '%s'", type)
 
@@ -478,9 +552,16 @@ class DeviantartExtractor(Extractor):
             for mark in marks:
                 type = mark["type"]
                 if type == "link":
+                    attrs = mark.get("attrs") or {}
                     html.append('<a href="')
-                    html.append(text.escape(mark["attrs"]["href"]))
-                    html.append('" rel="noopener noreferrer nofollow ugc">')
+                    html.append(text.escape(attrs.get("href") or ""))
+                    if "target" in attrs:
+                        html.append('" target="')
+                        html.append(attrs["target"])
+                    html.append('" rel="')
+                    html.append(attrs.get("rel") or
+                                "noopener noreferrer nofollow ugc")
+                    html.append('">')
                     close.append("</a>")
                 elif type == "bold":
                     html.append("<strong>")
@@ -491,6 +572,9 @@ class DeviantartExtractor(Extractor):
                 elif type == "underline":
                     html.append("<u>")
                     close.append("</u>")
+                elif type == "strike":
+                    html.append("<s>")
+                    close.append("</s>")
                 elif type == "textStyle" and len(mark) <= 1:
                     pass
                 else:
@@ -501,6 +585,18 @@ class DeviantartExtractor(Extractor):
         else:
             html.append(text.escape(content["text"]))
 
+    def _tiptap_process_children(self, html, content):
+        children = content.get("content")
+        if children:
+            for block in children:
+                self._tiptap_process_content(html, block)
+
+    def _tiptap_process_indentation(self, html, attrs):
+        itype = ("text-indent" if attrs.get("indentType") == "line" else
+                 "margin-inline-start")
+        isize = str((attrs.get("indentation") or 0) * 24)
+        html.append(itype + ":" + isize + "px")
+
     def _tiptap_process_deviation(self, html, content):
         dev = content["attrs"]["deviation"]
         media = dev.get("media") or ()
@@ -726,7 +822,7 @@ x2="45.4107524%" y2="71.4898596%" id="app-root-3">\
                 username, folder["gallery_id"], public=False):
             cache[dev["deviationid"]] = dev if has_access else None
 
-        return cache[deviation["deviationid"]]
+        return cache.get(deviation["deviationid"])
 
     def _unwatch_premium(self):
         for username in self.unwatch:
@@ -734,19 +830,22 @@ x2="45.4107524%" y2="71.4898596%" id="app-root-3">\
             self.api.user_friends_unwatch(username)
 
     def _eclipse_media(self, media, format="preview"):
-        url = [media["baseUri"], ]
+        url = [media["baseUri"]]
 
         formats = {
             fmt["t"]: fmt
             for fmt in media["types"]
         }
 
-        tokens = media["token"]
-        if len(tokens) == 1:
-            fmt = formats[format]
-            url.append(fmt["c"].replace("<prettyName>", media["prettyName"]))
-        url.append("?token=")
-        url.append(tokens[-1])
+        tokens = media.get("token") or ()
+        if tokens:
+            if len(tokens) <= 1:
+                fmt = formats[format]
+                if "c" in fmt:
+                    url.append(fmt["c"].replace(
+                        "<prettyName>", media["prettyName"]))
+            url.append("?token=")
+            url.append(tokens[-1])
 
         return "".join(url), formats
 
@@ -1144,7 +1243,6 @@ class DeviantartScrapsExtractor(DeviantartExtractor):
     subcategory = "scraps"
     directory_fmt = ("{category}", "{username}", "Scraps")
     archive_fmt = "s_{_username}_{index}.{extension}"
-    cookies_domain = ".deviantart.com"
     pattern = BASE_PATTERN + r"/gallery/(?:\?catpath=)?scraps\b"
     example = "https://www.deviantart.com/USER/gallery/scraps"
 
@@ -1161,7 +1259,6 @@ class DeviantartSearchExtractor(DeviantartExtractor):
     subcategory = "search"
     directory_fmt = ("{category}", "Search", "{search_tags}")
     archive_fmt = "Q_{search_tags}_{index}.{extension}"
-    cookies_domain = ".deviantart.com"
     pattern = (r"(?:https?://)?www\.deviantart\.com"
                r"/search(?:/deviations)?/?\?([^#]+)")
     example = "https://www.deviantart.com/search?q=QUERY"
@@ -1213,7 +1310,6 @@ class DeviantartGallerySearchExtractor(DeviantartExtractor):
     """Extractor for deviantart gallery searches"""
     subcategory = "gallery-search"
     archive_fmt = "g_{_username}_{index}.{extension}"
-    cookies_domain = ".deviantart.com"
     pattern = BASE_PATTERN + r"/gallery/?\?(q=[^#]+)"
     example = "https://www.deviantart.com/USER/gallery?q=QUERY"
 
@@ -1301,7 +1397,7 @@ class DeviantartOAuthAPI():
 
         metadata = extractor.config("metadata", False)
         if not metadata:
-            metadata = bool(extractor.extra)
+            metadata = True if extractor.extra else False
         if metadata:
             self.metadata = True
 
