@@ -28,8 +28,13 @@ class TwibooruExtractor(BooruExtractor):
 
     def _init(self):
         self.api = TwibooruAPI(self)
+        if not self.config("svg", True):
+            self._file_url = operator.itemgetter("view_url")
 
-    _file_url = operator.itemgetter("view_url")
+    def _file_url(self, post):
+        if post["format"] == "svg":
+            return post["view_url"].rpartition(".")[0] + ".svg"
+        return post["view_url"]
 
     @staticmethod
     def _prepare(post):
