@@ -38,7 +38,7 @@ class PathFormat():
                 filename_fmt = extractor.filename_fmt
             elif isinstance(filename_fmt, dict):
                 self.filename_conditions = [
-                    (util.compile_expression(expr),
+                    (util.compile_filter(expr),
                      formatter.parse(fmt, kwdefault).format_map)
                     for expr, fmt in filename_fmt.items() if expr
                 ]
@@ -57,7 +57,7 @@ class PathFormat():
                 directory_fmt = extractor.directory_fmt
             elif isinstance(directory_fmt, dict):
                 self.directory_conditions = [
-                    (util.compile_expression(expr), [
+                    (util.compile_filter(expr), [
                         formatter.parse(fmt, kwdefault).format_map
                         for fmt in fmts
                     ])
@@ -184,29 +184,31 @@ class PathFormat():
     def set_directory(self, kwdict):
         """Build directory path and create it if necessary"""
         self.kwdict = kwdict
-        sep = os.sep
 
         segments = self.build_directory(kwdict)
         if segments:
             self.directory = directory = self.basedirectory + self.clean_path(
-                sep.join(segments) + sep)
+                os.sep.join(segments) + os.sep)
         else:
             self.directory = directory = self.basedirectory
 
         if WINDOWS and self.extended:
-            # Enable longer-than-260-character paths
-            directory = os.path.abspath(directory)
-            if directory.startswith("\\\\"):
-                directory = "\\\\?\\UNC\\" + directory[2:]
-            else:
-                directory = "\\\\?\\" + directory
-
-            # abspath() in Python 3.7+ removes trailing path separators (#402)
-            if directory[-1] != sep:
-                directory += sep
-
+            directory = self._extended_path(directory)
         self.realdirectory = directory
 
+    def _extended_path(self, path):
+        # Enable longer-than-260-character paths
+        path = os.path.abspath(path)
+        if not path.startswith("\\\\"):
+            path = "\\\\?\\" + path
+        elif not path.startswith("\\\\?\\"):
+            path = "\\\\?\\UNC\\" + path[2:]
+
+        # abspath() in Python 3.7+ removes trailing path separators (#402)
+        if path[-1] != os.sep:
+            return path + os.sep
+        return path
+
     def set_filename(self, kwdict):
         """Set general filename data"""
         self.kwdict = kwdict
@@ -340,15 +342,22 @@ class PathFormat():
                 try:
                     os.replace(self.temppath, self.realpath)
                 except FileNotFoundError:
-                    # delayed directory creation
-                    os.makedirs(self.realdirectory)
+                    try:
+                        # delayed directory creation
+                        os.makedirs(self.realdirectory)
+                    except FileExistsError:
+                        # file at self.temppath does not exist
+                        return False
                     continue
                 except OSError:
                     # move across different filesystems
                     try:
                         shutil.copyfile(self.temppath, self.realpath)
                     except FileNotFoundError:
-                        os.makedirs(self.realdirectory)
+                        try:
+                            os.makedirs(self.realdirectory)
+                        except FileExistsError:
+                            return False
                         shutil.copyfile(self.temppath, self.realpath)
                     os.unlink(self.temppath)
                 break
